isNullable property
bool
get
isNullable
Returns true when the field is a nullable type
Implementation
bool get isNullable =>
fieldElementType.nullabilitySuffix == NullabilitySuffix.question;
Returns true when the field is a nullable type
bool get isNullable =>
fieldElementType.nullabilitySuffix == NullabilitySuffix.question;