pascalName property
String
get
pascalName
Returns the name of the field using a Pascal Case
Implementation
String get pascalName {
return name.pascalName;
}
Returns the name of the field using a Pascal Case
String get pascalName {
return name.pascalName;
}